library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity bus_select_v is

port (
    SYS_CLK         : in STD_LOGIC;
    SYS_RST_IN      : in STD_LOGIC;
    BUS_REL         : in STD_LOGIC;
    SYS_RST         : out STD_LOGIC;

    SDRAM_ADR       : in STD_LOGIC_VECTOR(0 TO 12);
    SDRAM_DQ_O      : in STD_LOGIC_VECTOR(0 TO 15);
    SDRAM_DQ_I      : out STD_LOGIC_VECTOR(0 TO 15);
    SDRAM_DQ_T      : in STD_LOGIC_VECTOR(0 TO 15);

    MEMCON_CEn      : in STD_LOGIC_VECTOR(0 TO 1);
    MEMCON_ADR      : in STD_LOGIC_VECTOR(0 TO 31);
    MEMCON_BEn      : in STD_LOGIC_VECTOR(0 TO 1);
    MEMCON_WEn      : in STD_LOGIC;
    MEMCON_OEn      : in STD_LOGIC_VECTOR(0 TO 1);

    MAC_WRn         : out STD_LOGIC;
    MAC_RDn         : out STD_LOGIC;
    MAC_BEn         : out STD_LOGIC_VECTOR(0 TO 1);  

    MEMCON_DQ_O     : in STD_LOGIC_VECTOR(0 TO 15);
    MEMCON_DQ_I     : out STD_LOGIC_VECTOR(0 TO 15);
    MEMCON_DQ_T     : in STD_LOGIC_VECTOR(0 TO 15);

    LA              : out STD_LOGIC_VECTOR(0 TO 22);
    LA10_RAM        : out STD_LOGIC;

    LD_O            : out STD_LOGIC_VECTOR(0 TO 15);
    LD_I            : in STD_LOGIC_VECTOR(0 TO 15);
    LD_T            : out STD_LOGIC_VECTOR(0 TO 15);

    FLASH_CEn       : out STD_LOGIC;
    FLASH_OEn       : out STD_LOGIC;
    FLASH_WEn       : out STD_LOGIC;

    BUS_REQ         : out STD_LOGIC;

    GPIO_IO_0_O     : in STD_LOGIC_VECTOR(0 TO 7);
    GPIO_IO_0_I     : out STD_LOGIC_VECTOR(0 TO 7);
    BOOTMODE            : in STD_LOGIC;
--  FPGA_RESET_EN   : inout STD_LOGIC;
    FPGA_RESET_EN_O : out STD_LOGIC;
    FPGA_RESET_EN_I : in STD_LOGIC;
    FPGA_RESET_EN_T : out STD_LOGIC

);
end bus_select_v;

architecture IMP of bus_select_v is

    signal MEMCON_OEn_w      : STD_LOGIC;
    signal MEMCON_WEn_w      : STD_LOGIC;
    signal MEMCON_CEn_w      : STD_LOGIC;
    signal MEMCON_CEn_reg    : STD_LOGIC;
    signal memcon_adr_change : STD_LOGIC_VECTOR(0 to 3);
    signal SYS_RST_w         : STD_LOGIC;
    signal LA_w              : STD_LOGIC_VECTOR(0 TO 22);
    signal counter : STD_LOGIC_VECTOR(0 to 7);
    signal reset_req_lat : STD_LOGIC;
    signal Bus_Reset : STD_LOGIC;
    signal Config_Reset : STD_LOGIC;

begin
-- System reset generate
SYS_RST_w <= SYS_RST_IN or not BUS_REL or Bus_Reset;
SYS_RST <= SYS_RST_w;

-- Flash control tri state buffer
FLASH_0 : OBUFT
port map (
    O => FLASH_CEn,
    I => MEMCON_CEn(1),
    T => SYS_RST_w
);
FLASH_1 : OBUFT
port map (
    O => FLASH_OEn,
    I => MEMCON_OEn_w,
    T => SYS_RST_w
);
FLASH_2 : OBUFT
port map (
    O => FLASH_WEn,
    I => MEMCON_WEn_w,
    T => SYS_RST_w
);

-- Address bus select
MEMCON_CEn_p : process (SYS_CLK, SYS_RST_w)
begin
    if SYS_RST_w = '1' then 
        MEMCON_CEn_reg <= '1';
    elsif SYS_CLK'event and SYS_CLK = '1' then
        MEMCON_CEn_reg <= not (not MEMCON_CEn(0) or not MEMCON_CEn(1));
    end if;
end process MEMCON_CEn_p;

MEMCON_OEn_w <= not (not MEMCON_OEn(1) and not MEMCON_CEn_reg);
MEMCON_WEn_w <= not (not MEMCON_WEn and not MEMCON_CEn_reg);
MEMCON_CEn_w <= not (not MEMCON_CEn(0) or not MEMCON_CEn(1) or not MEMCON_CEn_reg);

LA_w(22)       <= SDRAM_ADR(12);
LA_w(13 to 21) <= MEMCON_ADR(22 to 30) when (MEMCON_CEn_w = '0')
             else SDRAM_ADR(3 to 11) ;
LA_w(12)       <= MEMCON_ADR(21);
LA10_RAM       <= SDRAM_ADR(2);
LA_w(10 to 11) <= MEMCON_ADR(19 to 20) when (MEMCON_CEn_w = '0')
             else SDRAM_ADR(0 to 1) ;
LA_w(0 to 9)   <= MEMCON_ADR(9 to 18) ;

LA_GEN : for i in 0 to 22 generate
    OBUFT_0 : OBUFT
        port map (
            O => LA(i),
            I => LA_w(i),
            T => SYS_RST_w
        );
end generate;

-- data bus select
LD_O <= MEMCON_DQ_O(0 to 15) when MEMCON_DQ_T(15) = '0' or MEMCON_DQ_T(7) = '0' else SDRAM_DQ_O;

LD_GEN : for i in 0 to 15 generate
    LD_T(i) <= (MEMCON_DQ_T(i) and SDRAM_DQ_T(i)) or SYS_RST_w;
end generate;

SDRAM_DQ_I <= LD_I;
MEMCON_DQ_I(0 to 15) <= LD_I;

-- Mac control signal
MAC_WRn <= not (not MEMCON_WEn  and  not MEMCON_CEn(0) and not MEMCON_CEn_reg);
MAC_RDn <= not (not MEMCON_OEn(0)  and  not MEMCON_CEn(0) and not MEMCON_CEn_reg);
MAC_BEn <= MEMCON_BEn;

-- TE7720 control signal
BUS_REQ <= '1';

-- GPIO
FPGA_RESET_EN_O <= Config_Reset;
FPGA_RESET_EN_T <= SYS_RST_IN;
GPIO_IO_0_I <= "00000" & BOOTMODE & '0' &  FPGA_RESET_EN_I;


    process(SYS_CLK,SYS_RST_IN)
    begin
        if SYS_RST_IN = '1' then
            counter <= (others => '0');
        elsif SYS_CLK'event and SYS_CLK = '1' then 
            if reset_req_lat = '1' then
                counter <= counter + 1;
            else
                counter <= (others => '0');
            end if;
        end if;
    end process;

    process(SYS_CLK,SYS_RST_IN)
    begin
        if SYS_RST_IN = '1' then
            reset_req_lat <= '0';
            Bus_Reset <= '0';
            Config_Reset <= '0';
        elsif SYS_CLK'event and SYS_CLK = '1' then 
            reset_req_lat <= GPIO_IO_0_O(7) or reset_req_lat;
            Bus_Reset <= counter(1) or Bus_Reset;
            Config_Reset <= counter(0) or Config_Reset;
        end if;
    end process;

end IMP;
